uses
  sdl;
  
var
  window:pSDL_Surface;

procedure xdwrap_window_setcaption_sdl(title:string);
begin
  SDL_WM_SetCaption(pchar(title),nil);
end;

procedure xdwrap_window_seticon_sdl(filename:string);
var
  colorkey:cardinal;
  image:pSDL_Surface;
  err:xdwrap_Window_Error;
begin
  image:=SDL_LoadBMP(pchar(filename));
  if not assigned(image) then
  begin
    err:=xdwrap_Window_Error.create(pchar(SDL_GetError()));
    raise err;
    err.free;
    exit;
  end;
  colorkey:=SDL_MapRGB(image.format,0,0,0);
  SDL_SetColorKey(image,SDL_SRCCOLORKEY,colorkey);
  SDL_WM_SetIcon(image,0);
  sdl_freesurface(image);
end;

function xdwrap_window_getcaption_sdl:string;
var
  title,ico:pchar;
begin
  SDL_WM_GetCaption(title,ico);
  result:=title;
end;

function xdwrap_window_geticon_sdl:string;
var
  title,ico:pchar;
begin
  SDL_WM_GetCaption(title,ico);
  result:=ico;
end;

function xdwrap_window_init_sdl:integer;
begin
  result:=sdl_init(SDL_INIT_VIDEO);
end;

function xdwrap_window_create_sdl(width, height : integer; depth, flags : cardinal):boolean;
var
   sdlflags : cardinal;
   VidInfo  : PSDL_VideoInfo;
begin
  VidInfo:= SDL_GetVideoInfo;
	case depth of
    16:begin
         SDL_GL_SetAttribute(SDL_GL_RED_SIZE,   05);
			   SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 05);
         SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,  05);
			   SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 01);
	     end;
		24:begin
			   SDL_GL_SetAttribute(SDL_GL_RED_SIZE,   06);
			   SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 06);
			   SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,  06);
			   SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 06);
	     end;
		32:begin
			   SDL_GL_SetAttribute(SDL_GL_RED_SIZE,   08);
			   SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 08);
			   SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,  08);
			   SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 08);
	     end;
	end;

  SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE,  depth);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,   16);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 01);

  if VidInfo.hw_available<>0 then
    sdlflags:= SDL_HWSURFACE
  else
    sdlflags:= SDL_SWSURFACE;

  if VidInfo.blit_hw<>0 then 
    sdlflags:= sdlflags or SDL_HWACCEL;

  sdlflags:= sdlflags or SDL_DOUBLEBUF or SDL_HWPALETTE or SDL_OPENGL;

  case flags of
    WND_TYPE_DIALOG     : sdlflags:=  sdlflags and not(SDL_NOFRAME or SDL_RESIZABLE or SDL_FULLSCREEN);
    WND_TYPE_FULLSCREEN : sdlflags:= (sdlflags or SDL_FULLSCREEN) and not(SDL_NOFRAME or SDL_RESIZABLE);
    WND_TYPE_NOFRAME    : sdlflags:= (sdlflags or SDL_NOFRAME) and not (SDL_RESIZABLE or SDL_FULLSCREEN);
    WND_TYPE_RESIZEABLE : sdlflags:= (sdlflags or SDL_RESIZABLE) and not (SDL_NOFRAME or SDL_FULLSCREEN);
  end;

  window:= SDL_SetVideoMode(width,height,depth,sdlflags);
  if assigned(window) then
    result:= true
  else
    result:= false;
end;

function xdwrap_window_togglefullscreen_sdl():Boolean;
var
  flags,depth:cardinal;
  w,h:integer;
begin
  {$IFDEF WIN32}
    w:=window.w;
    h:=window.h;
    depth:=window.format.bitsperpixel;
    flags:=window.flags;

    if flags and SDL_FULLSCREEN=SDL_FULLSCREEN then
      flags:=flags and not SDL_FULLSCREEN
    else
      flags:=flags or SDL_FULLSCREEN;

    window:=SDL_SetVideoMode(w,h,depth,flags);
    if assigned(window) then
      result:=true
    else
      result:=false;
  {$ELSE}
  if SDL_WM_ToggleFullScreen(window)=0 then
    result:=false
  else
    result:=true;
  {$ENDIF}
end;

function xdwrap_window_minimize_sdl:Boolean;
begin
  if SDL_WM_IconifyWindow()=0 then
    result:=false
  else
    result:=true;
end;

procedure xdwrap_window_fixmouse_sdl(fixmode:Boolean);
begin
  if fixmode then
    SDL_WM_GrabInput(SDL_GRAB_ON)
  else
    SDL_WM_GrabInput(SDL_GRAB_OFF);
end;

procedure xdwrap_window_swapbuffer_sdl;
begin
  SDL_GL_SwapBuffers;
end;

procedure init_window;
begin
  xdwrap_window_setcaption:=xdwrap_window_setcaption_sdl;
  xdwrap_window_seticon:=xdwrap_window_seticon_sdl;
  xdwrap_window_getcaption:=xdwrap_window_getcaption_sdl;
  xdwrap_window_geticon:=xdwrap_window_geticon_sdl;
  xdwrap_window_init:=xdwrap_window_init_sdl;
  xdwrap_window_create:=xdwrap_window_create_sdl;
  xdwrap_window_togglefullscreen:=xdwrap_window_togglefullscreen_sdl;
  xdwrap_window_minimize:=xdwrap_window_minimize_sdl;
  xdwrap_window_fixmouse:=xdwrap_window_fixmouse_sdl;
  xdwrap_window_swapbuffer:=xdwrap_window_swapbuffer_sdl;
end;